/*
          LTR25.
            ,    
           :
    ltr25_recv  slot crate_serial srvip
    :
        slot         -   ( 1  16)
        crate_serial -    (   )
        srvip        - ip-  ltrd  LtrServer (     )
     :    ,       

              .
        - ( - )   
      .

         ,   
       .

           Windows   CTRL+C  Linux

      VisualStudio:
          Visual Studio,     
    ( (Project) ->  (Properties) ->   (Configuration Properties)
    -> /++ ->  (General) ->    (Additional Include Directories))
            (ltr25api.h    
         ltr/include   )
        .lib   <  >/lib/msvc
    ( (Project) ->  (Properties) ->   (Configuration Properties) ->
     (Linker) ->  (General) ->    (Additional Library Directories)).

    !:      Visual Studio    
       ,    :
      (File) ->    (Advanced Save Options)...
        (Encoding)   (UTF8,  )/Unicode (UTF-8 with signature)
        .
*/

#include "ltr/include/ltr25api.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef _WIN32
#include <locale.h>
#include <conio.h>
#else
#include <signal.h>
#include <unistd.h>
#endif


/*    ,    */
#define RECV_BLOCK_CH_SIZE  4096*16
/*       */
#define RECV_TOUT  4000


typedef struct {
    int slot;
    const char *serial;
    DWORD addr;
} t_open_param;


/*      */
static int f_out = 0;

#ifndef _WIN32
/*     Linux */
static void f_abort_handler(int sig) {
    f_out = 1;
}
#endif


/*    .   ,  
 *   :
 * 1  -   ( 1  16)
 * 2  -   
 * 3  - ip-  */
static int f_get_params(int argc, char** argv, t_open_param* par) {
    int err = 0;
    par->slot = LTR_CC_CHNUM_MODULE1;
    par->serial = "";
    par->addr = LTRD_ADDR_DEFAULT;


    if (argc > 1)
        par->slot = atoi(argv[1]);
    if (argc > 2)
        par->serial = argv[2];
    if (argc > 3) {
        int a[4],i;
        if (sscanf(argv[3], "%d.%d.%d.%d", &a[0], &a[1], &a[2], &a[3])!=4) {
            fprintf(stderr, "  IP-!!\n");
            err = -1;
        }

        for (i=0; (i < 4) && !err; i++) {
            if ((a[i]<0) || (a[i] > 255)) {
                fprintf(stderr, " IP-!!\n");
                err = -1;
            }
        }

        if (!err) {
            par->addr = (a[0] << 24) | (a[1]<<16) | (a[2]<<8) | a[3];
        }
    }
    return err;
}

int main(int argc, char** argv) {
    INT err = LTR_OK;
    TLTR25 hltr25;
    t_open_param par;
#ifndef _WIN32
    struct sigaction sa;
    /*   Linux      ,
           */
    sa.sa_handler = f_abort_handler;
    sigaction(SIGTERM, &sa, NULL);
    sigaction(SIGINT, &sa, NULL);
    sigaction(SIGABRT, &sa, NULL);
#endif

#ifdef _WIN32
    /*         Windows*/
    setlocale(LC_ALL, "");
#endif

    err = f_get_params(argc, argv, &par);

    if (err==LTR_OK) {
        LTR25_Init(&hltr25);
        /*     */
        err = LTR25_Open(&hltr25, par.addr, LTRD_PORT_DEFAULT, par.serial, par.slot);
        if (err!=LTR_OK) {
            fprintf(stderr, "     .  %d (%s)\n",
                    err, LTR25_GetErrorString(err));
        } else {
            INT close_err;

            /*      */
            printf("  !   : \n");
            printf("       = %s\n", hltr25.ModuleInfo.Name);
            printf("        = %s\n", hltr25.ModuleInfo.Serial);
            printf("   PLD         = %d\n", hltr25.ModuleInfo.VerPLD);
            printf("           = %d\n", hltr25.ModuleInfo.VerFPGA);
            printf("         = %d\n", hltr25.ModuleInfo.BoardRev);
            printf("  .      = %s\n", hltr25.ModuleInfo.Industrial ? "" : "");
            fflush(stdout);

            /*   */
            /*  - 24  20  */
            hltr25.Cfg.DataFmt = LTR25_FORMAT_32;
            /*        */
            hltr25.Cfg.FreqCode = LTR25_FREQ_78K;
            /*   2.86  10  */
            hltr25.Cfg.ISrcValue = LTR25_I_SRC_VALUE_2_86;

            /*    */
            hltr25.Cfg.Ch[0].Enabled = TRUE;
            hltr25.Cfg.Ch[1].Enabled = TRUE;
            hltr25.Cfg.Ch[2].Enabled = TRUE;

            err = LTR25_SetADC(&hltr25);
            if (err!=LTR_OK) {
                fprintf(stderr, "    :  %d (%s)\n",
                        err, LTR25_GetErrorString(err));
            } else {
                /*  SetADC()   AdcFreq.   
                 *   */
                printf("   .  = %.2f , -  = %d\n",
                        hltr25.State.AdcFreq, hltr25.State.EnabledChCnt);
            }

            if (err==LTR_OK) {
                DWORD recvd_blocks=0;
                INT recv_data_cnt = RECV_BLOCK_CH_SIZE*hltr25.State.EnabledChCnt;
                /*  24-        ,
                     20- -  */
                INT   recv_wrd_cnt = recv_data_cnt*(hltr25.Cfg.DataFmt==LTR25_FORMAT_20 ? 1 : 2);
                DWORD  *rbuf = (DWORD*)malloc(recv_wrd_cnt*sizeof(rbuf[0]));
                double *data = (double *)malloc(recv_data_cnt*sizeof(data[0]));
                DWORD  *ch_status = (DWORD*)malloc(hltr25.State.EnabledChCnt*sizeof(ch_status[0]));

                if ((rbuf==NULL) || (data==NULL) || (ch_status==NULL)) {
                    fprintf(stderr, "  !\n");
                    err = LTR_ERROR_MEMORY_ALLOC;
                }

                if (err==LTR_OK) {
                    /*    */
                    err=LTR25_Start(&hltr25);
                    if (err!=LTR_OK) {
                        fprintf(stderr, "    !  %d (%s)\n",
                                err, LTR25_GetErrorString(err));
                    }
                }

                if (err==LTR_OK) {
                    printf("  .    %s\n",
#ifdef _WIN32
                           " "
#else
                           "CTRL+C"
#endif
                           );
                    fflush(stdout);
                }


                /*        
                 *    */
                while (!f_out && (err==LTR_OK)) {
                    INT recvd;
                    /*         */
                    DWORD tout = RECV_TOUT + (DWORD)(1000.*RECV_BLOCK_CH_SIZE/hltr25.State.AdcFreq + 1);

                    /*    .  */
                    recvd = LTR25_Recv(&hltr25, rbuf, NULL, recv_wrd_cnt, tout);

                    /*       */
                    if (recvd<0) {
                        err = recvd;
                        fprintf(stderr, "  .  %d:%s\n",
                                err, LTR25_GetErrorString(err));
                    } else if (recvd!=recv_wrd_cnt) {
                        fprintf(stderr, "  .  %d,  %d\n",
                                recv_wrd_cnt, recvd);
                        err = LTR_ERROR_RECV_INSUFFICIENT_DATA;
                    } else {
                        err = LTR25_ProcessData(&hltr25, rbuf, data, &recvd,
                                                LTR25_PROC_FLAG_VOLT, ch_status);
                        if (err!=LTR_OK) {
                            fprintf(stderr, "  .  %d:%s\n",
                                    err, LTR25_GetErrorString(err));
                        } else {
                            unsigned ch;
                            recvd_blocks++;
                            /*       */
                            printf(" %4d: ", recvd_blocks);

                            for (ch=0; ch < hltr25.State.EnabledChCnt; ch++) {
                                if (ch_status[ch]==LTR25_CH_STATUS_OPEN) {
                                    printf("%10s", "");
                                } else if (ch_status[ch]==LTR25_CH_STATUS_SHORT) {
                                    printf("%10s", "");
                                } else if (ch_status[ch]==LTR25_CH_STATUS_OK) {
                                    INT i;
                                    double min_val, max_val, pp;
                                    /*      
                                     *    ( 
                                     *     LTR25) */
                                    min_val = max_val = data[ch];
                                    for (i=1; i <  recvd / hltr25.State.EnabledChCnt; i++) {
                                        double val = data[i*hltr25.State.EnabledChCnt+ch];
                                        if (min_val > val)
                                            min_val = val;
                                        if (max_val < val)
                                            max_val = val;
                                    }
                                    pp = (max_val - min_val);
                                    printf("%10.2f", pp);
                                } else {
                                    printf(". .!");
                                }

                                if (ch==(hltr25.State.EnabledChCnt-1U)) {
                                    printf("\n");
                                } else {
                                    printf(",  ");
                                }
                            }
                            fflush(stdout);

                        }
                    }

#ifdef _WIN32
                    /*      */
                    if (err==LTR_OK) {
                        if (_kbhit())
                            f_out = 1;
                    }
#endif
                } //while (!f_out && (err==LTR_OK))




                /*    ,    */
                if (hltr25.State.Run) {
                    INT stop_err = LTR25_Stop(&hltr25);
                    if (stop_err!=LTR_OK) {
                        fprintf(stderr, "    .  %d:%s\n",
                                stop_err, LTR25_GetErrorString(stop_err));
                        if (err==LTR_OK)
                            err = stop_err;
                    } else {
                        printf("  .\n");
                    }
                }


                free(rbuf);
                free(data);
                free(ch_status);
            }

            /*     */
            close_err = LTR25_Close(&hltr25);
            if (close_err!=LTR_OK) {
                fprintf(stderr, "     .  %d:%s\n",
                        close_err, LTR25_GetErrorString(close_err));
                if (err==LTR_OK)
                    err = close_err;
            } else {
                printf("    .\n");
            }
        }
    }

    return err;
}
